#include <d3d9.h>
#include <d3dx9.h>
#include "stdio.h"

//Simple structure for managing multiple shaders.
typedef struct VERTEX_SHADER_INFO
{
	char                   *pName;
	char                   *pFileName;
	char                   *pTargetName;
	LPDIRECT3DVERTEXSHADER9 pShader;
} VERTEX_SHADER_INFO;

typedef struct PIXEL_SHADER_INFO
{
	char                   *pName;
	char                   *pFileName;
	char                   *pTargetName;
	LPDIRECT3DPIXELSHADER9 pShader;
} PIXEL_SHADER_INFO;

typedef struct MODEL_INFO
{
	char                   *pName;
	char                   *pFileName;
	LPD3DXMESH              pModel;
	LPDIRECT3DVERTEXBUFFER9 pModelVertices;
	LPDIRECT3DINDEXBUFFER9  pModelIndices;
	long                    NumFaces;
	long                    NumVertices;
} MODEL_INFO;

//make sure this number is 1+ the number of shaders 
//you want to load. I do this so that a check of the 
//shader name checks both a valid shader and the end of
//the array.
#define NUM_VERTEX_SHADERS 64 //more than enough...
#define NUM_PIXEL_SHADERS  64

//make sure this number is 1+ the number of models 
//you want to load.
#define NUM_MODELS 64

//Make sure that keys aren't overly responsive
#define CHANGE_DELAY 500

class CSampleApplication
{
public:
	CSampleApplication();
	virtual ~CSampleApplication();
	LRESULT WINAPI MessageHandler(HWND hWnd, UINT msg, 
								  WPARAM wParam, LPARAM lParam);

	void Go();

protected:
	HRESULT InitializeD3D();
	HRESULT CreateDevice(D3DDEVICE_CREATION_PARAMETERS *pCreateParms, D3DPRESENT_PARAMETERS *pPresentParms);
	HRESULT CreateWindowed(HWND WindowHandle, D3DDEVTYPE DeviceType, DWORD Behavior);
	HRESULT RestoreDevice();
	HRESULT DestroyDevice();

	virtual BOOL Initialize();
	virtual BOOL Terminate();
	virtual void Render();

	void CreateShaders();
	void CreateVertices();

	virtual BOOL    HandleMessage(MSG *pMessage);
	virtual BOOL    PreReset();
	virtual BOOL    PostReset();
	virtual void    PreRender();
	virtual HRESULT PostRender();

	//These matrices and parameters are used to
	//transform the model.
	D3DXMATRIXA16 m_World;
	D3DXMATRIXA16 m_InverseWorld;
	D3DXMATRIXA16 m_View;
	D3DXMATRIXA16 m_Projection;
	D3DXMATRIXA16 m_ShaderMatrix;
	float         m_FOV;
	float         m_AspectRatio;
	float         m_WorldAngle;

	//These members maintain the vertex information
	LPDIRECT3DVERTEXDECLARATION9 m_pVertexDeclaration;
	D3DVERTEXELEMENT9           *m_pVertexElements;
	long                         m_Stride;

	//These members manage the shaders
	VERTEX_SHADER_INFO m_VertexShaders[NUM_VERTEX_SHADERS];
	long               m_CurrentVertexShader;

	PIXEL_SHADER_INFO  m_PixelShaders[NUM_PIXEL_SHADERS];
	long               m_CurrentPixelShader;

	//These members manage the models
	MODEL_INFO m_Models[NUM_MODELS];
	long       m_CurrentModel;
	HWND m_hWnd;
	long m_WindowWidth;
	long m_WindowHeight;

	LPDIRECT3DDEVICE9 m_pD3DDevice;
	LPDIRECT3D9       m_pD3D;
	D3DPRESENT_PARAMETERS         m_PresentParameters;
	D3DDEVICE_CREATION_PARAMETERS m_CreationParameters;

	BOOL m_Continue;
	long m_LastChangeTime;

	char m_TextBuffer[1000];
	LPD3DXFONT m_pFont;
	RECT m_TextRect;

	__int64 m_HighResFrequency;
};
